/**********************************************************************
 * str.c                                                    August 1999
 * Horms                                             horms@valinux.com.
 * VA Linux System                                 http://valinux.com./
 *
 * Various string handling functions
 *
 * Licenced under the terms of the GNU GPL
 *
 **********************************************************************/

#include "str.h"

/**********************************************************************
 * str_is_digit
 * Test if a null terminated string is composed entirely of digits (0-9)
 *
 * Pre: String
 * Post: 1 if string contains only digits and null teminator
 *       0 otherwise
 *
 * Not 8 bit clean
 **********************************************************************/

int str_is_digit(const char *str){
  int offset;

  for(offset=strlen(str)-1;offset>-1;offset--){
    if(!isdigit(*(str+offset))){
      break;
    }
  }

  return(offset>-1?0:1);
}

/**********************************************************************
 * strn_to_str
 * Pre: dest: memory to put new string in
 *      src:  source string
 *      n:    bytes from src to put in dest 
 *            a null character will be added to the end of dest
 *            hence, dest will be n+1 bytes long
 * Return: NULL on error
 *         src otherwise
 *
 * Not 8 bit clean
 **********************************************************************/

char *strn_to_str(char *dest, const char *src, size_t n){
  if((dest=(char *)malloc(n+1))==NULL){
    syslog(
      LOG_WARNING, 
      "strn_to_str: malloc",
      strerror(errno)
    );
    return(NULL);
  }
  strncpy(dest, src, n);
  *(dest+n)='\0';

  return(dest);
}

/**********************************************************************
 * write_strn
 * write a n bytes of a string to fd
 *
 * Pre: fd: File descriptor to write to
 *      string: string to write
 *      n: number or bytes to write
 * Return: -1 on error
 *         0 otherwise
 *
 * 8 bit clean
 **********************************************************************/

int write_strn(const int fd, const unsigned char *string, const ssize_t n){
  ssize_t offset;
  ssize_t bytes_written;

  if(n==0){
   return(0);
  }

  offset=0;
  do{
    bytes_written=write(fd, string+offset, n-offset);
    if(bytes_written<0){
      syslog(
        LOG_WARNING, 
        "write_strn: error writing output: %s",
        strerror(errno)
      );
      return(-1);
    }
    offset+=bytes_written;
  }while(offset<n);
  
  return(0);
}


/**********************************************************************
 * write_str
 * write strings to fd by puting them into tokens and
 * printing the tokens
 * append a CRLF to the output (intput strings should not end in a CRLF)
 * 
 * Pre: fd: File descriptor to write to
 *      nostring: number of strings
 *      ...: strings
 * Return: -1 on error
 *         0 otherwise
 *
 * Not 8 bit clean
 **********************************************************************/

int write_str(const int fd, int nostring, ...){
  va_list ap;
  char *string;

  if(nostring<1){
    return(1);
  }

  va_start(ap, nostring);
  while(nostring-->0){
    string=va_arg(ap, char*);
    if(string==NULL){
      syslog(LOG_WARNING, "write_str: null string");
      return(-1);
    }
    if(write_strn(fd, (unsigned char *)string, (ssize_t)strlen(string))<0){
      syslog(LOG_WARNING, "write_str: write_strn");
      return(-1);
    }
  }
  va_end(ap);

  if(write_strn(fd, (unsigned char *)"\r\n", (ssize_t)2)<0){
    syslog(LOG_WARNING, "write_str: write_strn");
    return(-1);
  }
  return(0);
}


/**********************************************************************
 * cat_str
 * 
 * Pre: dest: Unallocated string to place result in
 *      nostring: number of strings
 *      ...: strings
 * Return: NULL on error
 *         concatenated string otherwise
 *
 * Not 8 bit clean
 **********************************************************************/

char *cat_str(char *dest, int nostring, ...){
  va_list ap;
  char **string;
  char **current_string;
  int length;
  int i;

  if(nostring<1){
    return(NULL);
  }

  if((string=(char **)malloc(sizeof(char *)*nostring))==NULL){
    syslog(LOG_WARNING, "cat_str: malloc 1");
    return(NULL);
  }

  current_string=string;
  length=1;

  va_start(ap, nostring);
  for(i=0;i<nostring;i++){
    *current_string=va_arg(ap, char*);
    if(*current_string==NULL){
      syslog(LOG_WARNING, "write_str: null string");
      return(NULL);
    }
    length+=strlen(*current_string);
    current_string++;
  }
  va_end(ap);

  if((dest=(char *)malloc(sizeof(char)*length))==NULL){
    syslog(LOG_WARNING, "cat_str: malloc 2");
    return(NULL);
  }

  current_string=string;
  strcpy(dest, *current_string++);
  for(i=1;i<nostring;i++){
    strcat(dest, *current_string++);
  }

  free(string);

  return(dest);
}


/**********************************************************************
 * str_free
 * Free a string if it is nut NULL
 *
 * Not 8 bit clean
 **********************************************************************/

void str_free(char **string){
  if(*string!=NULL){
    free(*string);
    *string=NULL;
  }
}

